@@ -66,11 +66,10 @@ int attribs_encode(struct sbuf *sb)
 		// Protocol1 does not have this field.
 		p += to_base64(sb->protocol2->index, p);
 		*p++ = ' ';
-		// Protocol2 puts compression near the beginning.
+		// Protocol2 puts compression/encryption near the beginning.
 		p += to_base64(sb->compression, p);
 		*p++ = ' ';
-		// Protocol1 does not have this field.
-		p += to_base64(sb->protocol2->encryption, p);
+		p += to_base64(sb->encryption, p);
 		*p++ = ' ';
 	}
 	p += to_base64(statp->st_dev, p);
@@ -118,9 +117,13 @@ int attribs_encode(struct sbuf *sb)
 
 	if(sb->protocol1)
 	{
-		// Protocol1 puts compression at the end.
+		// Protocol1 puts compression/encryption at the end.
 		*p++ = ' ';
 		p += to_base64(sb->compression, p);
+		*p++ = ' ';
+		p += to_base64(sb->encryption, p);
+		*p++ = ' ';
+		p += to_base64(sb->protocol1->salt, p);
 	}
 
 	*p = 0;
@@ -153,9 +156,9 @@ void attribs_decode(struct sbuf *sb)
 		p += from_base64(&val, p);
 		sb->compression=val;
 		p++;
-		// Protocol1 does not have this field.
+		// Encryption for protocol2.
 		p += from_base64(&val, p);
-		sb->protocol2->encryption=val;
+		sb->encryption=val;
 		p++;
 	}
 	p += from_base64(&val, p);
@@ -220,18 +223,20 @@ void attribs_decode(struct sbuf *sb)
 	}
 
 	// Look for winattr.
+	sb->winattr=0;
 	if(*p == ' ' || (*p != 0 && *(p+1) == ' '))
 	{
 		p++;
 		p += from_base64(&val, p);
+		sb->winattr=val;
 	}
-	else
-		val = 0;
-	sb->winattr=val;
 
-	// Compression for protocol1.
 	if(sb->protocol1)
 	{
+		sb->compression=-1;
+		sb->encryption=ENCRYPTION_UNSET;
+
+		// Compression for protocol1.
 		if(*p == ' ' || (*p != 0 && *(p+1) == ' '))
 		{
 			p++;
@@ -239,8 +244,24 @@ void attribs_decode(struct sbuf *sb)
 			p += from_base64(&val, p);
 			sb->compression=val;
 		}
-		else
-			sb->compression=-1;
+
+		// Encryption for protocol1.
+		if(*p == ' ' || (*p != 0 && *(p+1) == ' '))
+		{
+			p++;
+			if(!*p) return;
+			p += from_base64(&val, p);
+			sb->encryption=val;
+		}
+
+		// Salt for protocol1.
+		if(*p == ' ' || (*p != 0 && *(p+1) == ' '))
+		{
+			p++;
+			if(!*p) return;
+			p += from_base64(&val, p);
+			sb->protocol1->salt=val;
+		}
 	}
 }
 
@@ -249,17 +270,29 @@ int attribs_set_file_times(struct asfd *asfd,
 	struct cntr *cntr)
 {
 	int e;
-	struct utimbuf ut;
-	ut.actime=statp->st_atime;
-	ut.modtime=statp->st_mtime;
 
-// The mingw64 utime() appears not to work on read-only files.
-// Use the utime() from bacula instead.
 #ifdef HAVE_WIN32
-	//e=utime(path, &ut);
-	e=win32_utime(path, &ut);
+	// You (probably) cannot set times on Windows junction points.
+	if(statp->st_rdev==WIN32_JUNCTION_POINT)
+		return 0;
+
+	// The mingw64 utime() appears not to work on read-only files.
+	// Use the utime() from bacula instead.
+	e=win32_utime(path, statp);
+#elif HAVE_LUTIMES
+	struct timeval t[2];
+	t[0].tv_sec = statp->st_atime;
+	t[0].tv_usec = 0;
+	t[1].tv_sec = statp->st_mtime;
+	t[1].tv_usec = 0;
+	e=lutimes(path, t);
 #else
-	e=utime(path, &ut);
+	struct timespec ts[2];
+	ts[0].tv_sec=statp->st_atime;
+	ts[0].tv_nsec=0;
+	ts[1].tv_sec=statp->st_mtime;
+	ts[1].tv_nsec=0;
+	e=utimensat(AT_FDCWD, path, ts, AT_SYMLINK_NOFOLLOW);
 #endif
 	if(e<0)
 	{
@@ -288,18 +321,6 @@ uint64_t decode_file_no_and_save_path(struct iobuf *iobuf, char **save_path)
 	return (uint64_t)val;
 }
 
-#ifdef HAVE_LUTIMES
-static int do_lutimes(const char *path, struct stat *statp)
-{
-	struct timeval t[2];
-	t[0].tv_sec = statp->st_atime;
-	t[0].tv_usec = 0;
-	t[1].tv_sec = statp->st_mtime;
-	t[1].tv_usec = 0;
-	return lutimes(path, t);
-}
-#endif
-
 int attribs_set(struct asfd *asfd, const char *path,
 	struct stat *statp, uint64_t winattr, struct cntr *cntr)
 {
@@ -313,10 +334,28 @@ int attribs_set(struct asfd *asfd, const char *path,
 	{
 		struct berrno be;
 		berrno_init(&be);
-		logw(asfd, cntr,
-			"Unable to set file owner of %s to %d:%d: ERR=%s\n",
+		char msg[256]="";
+
+		snprintf(msg, sizeof(msg),
+			"Unable to set file owner of %s to %d:%d: ERR=%s",
 			path, statp->st_uid, statp->st_gid,
 			berrno_bstrerror(&be, errno));
+
+		if(errno==EPERM)
+		{
+			static int do_owner_warning=1;
+			if(getuid()!=0)
+			{
+				if(!do_owner_warning)
+					return -1;
+
+				logw(asfd, cntr, "%s - possibly because you are not root. Will suppress subsequent messages of this type.\n", msg);
+				do_owner_warning=0;
+				return -1;
+			}
+		}
+		logw(asfd, cntr, "%s\n", msg);
+
 		return -1;
 	}
 
@@ -325,15 +364,8 @@ int attribs_set(struct asfd *asfd, const char *path,
 	   meta stuff on links. */
 	if(S_ISLNK(statp->st_mode))
 	{
-#ifdef HAVE_LUTIMES
-		if(do_lutimes(path, statp)) {
-			struct berrno be;
-			berrno_init(&be);
-			logw(asfd, cntr, "Unable to set lutimes %s: ERR=%s\n",
-				path, berrno_bstrerror(&be, errno));
+		if(attribs_set_file_times(asfd, path, statp, cntr))
 			return -1;
-		}
-#endif
 	}
 	else
 	{
