@@ -89,7 +89,7 @@ int extra_comms_client(struct async *as, struct conf **confs,
 				goto end;
 			if(*incexc)
 			{
-				if(conf_parse_incexcs_buf(confs, *incexc))
+				if(conf_parse_incexcs_srestore(confs, *incexc))
 					goto end;
 				*action=ACTION_RESTORE;
 				log_restore_settings(confs, 1);
@@ -234,7 +234,7 @@ int extra_comms_client(struct async *as, struct conf **confs,
 			goto end;
 	}
 
-#ifndef RS_DEFAULT_STRONG_LEN
+#ifdef HAVE_BLAKE2
 	if(server_supports(feat, ":rshash=blake2:"))
 	{
 		set_e_rshash(confs[OPT_RSHASH], RSHASH_BLAKE2);
