@@ -17,6 +17,7 @@
 #include "extra_comms.h"
 #include "list.h"
 #include "monitor.h"
+#include "find_logic.h"
 #include "monitor/status_client_ncurses.h"
 #include "protocol2/restore.h"
 #include "restore.h"
@@ -182,15 +183,20 @@ static enum cliret backup_wrapper(struct asfd *asfd,
 			args[a++]="reserved5";
 		}
 		args[a++]=NULL;
+		// At this point, the server may have closed the connection,
+		// so cannot log remotely.
 		if(run_script(asfd,
 			args, get_strlist(confs[OPT_B_SCRIPT_POST_ARG]),
-			confs, 1, 1, 1))
+			confs, 1, 1, /*log_remote*/ 0))
 				ret=CLIENT_ERROR;
 	}
 
 	if(ret==CLIENT_OK) logp("backup finished ok\n");
 
 end:
+	// The include_logic/exclude_logic cache may have been populated
+	// during backup so we clean it here
+	free_logic_cache();
 	return ret;
 error:
 	logp("error in backup\n");
@@ -386,7 +392,7 @@ static enum cliret restore_wrapper(struct asfd *asfd, enum action action,
 		args[a++]=NULL;
 		if(run_script(asfd,
 			args, get_strlist(confs[OPT_R_SCRIPT_POST_ARG]),
-			confs, 1, 1, 1))
+			confs, 1, 1, /*log_remote*/ 0))
 				ret=CLIENT_ERROR;
 	}
 
