@@ -619,7 +619,13 @@ int cntr_stats_to_file(struct cntr *cntr,
 
 	if(!(path=prepend_s(directory, fname)))
 		goto end;
-	if((fd=open(path, O_WRONLY|O_CREAT, 0666))<0)
+	if((fd=open(
+		path,
+#ifdef O_NOFOLLOW
+		O_NOFOLLOW|
+#endif
+		O_WRONLY|O_CREAT,
+		0666))<0)
 	{
 		logp("Could not open %s for writing in %s: %s\n",
 			path, __func__, strerror(errno));
@@ -629,14 +635,16 @@ int cntr_stats_to_file(struct cntr *cntr,
 	if(!(as=async_alloc())
 	  || as->init(as, 0)
 	  || !(wfd=setup_asfd_linebuf_write(as, "stats file", &fd)))
-			goto end;
+	{
+		close_fd(&fd);
+		goto end;
+	}
 
 	if(json_cntr(wfd, cntr))
 		goto end;
 
 	ret=0;
 end:
-	close_fd(&fd);
 	free_w(&path);
 	async_free(&as);
 	asfd_free(&wfd);
@@ -954,7 +962,7 @@ int cntr_send_sdirs(struct asfd *asfd,
 	// FIX THIS:
 	// It would be better just to set up the correct 'bu' entry instead
 	// of loading everything and then looking through the list.
-	if(bu_get_list_with_working(sdirs, &bu_list, NULL))
+	if(bu_get_list_with_working(sdirs, &bu_list))
 		goto end;
 	for(bu=bu_list; bu; bu=bu->next)
 		if((bu->flags & BU_WORKING)
@@ -1082,3 +1090,12 @@ const char *cntr_status_to_action_str(struct cntr *cntr)
 			return "unknown";
 	}
 }
+
+int check_fail_on_warning(int fail_on_warning, struct cntr_ent *warn_ent)
+{
+	if(!fail_on_warning || !warn_ent || !warn_ent->count)
+		return 0;
+	logp("fail_on_warning is set and warning count is %" PRIu64 "\n",
+		warn_ent->count);
+	return -1;
+}
