@@ -4,7 +4,6 @@
 #include "log.h"
 #include "alloc.h"
 #include "cntr.h"
-#include "strlist.h"
 #include "prepend.h"
 #include "server/dpth.h"
 
@@ -418,6 +417,8 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	  return sc_str(c[o], 0, 0, "ssl_ciphers");
 	case OPT_SSL_COMPRESSION:
 	  return sc_int(c[o], 5, 0, "ssl_compression");
+	case OPT_SSL_VERIFY_PEER_EARLY:
+	  return sc_int(c[o], 0, 0, "ssl_verify_peer_early");
 	case OPT_RATELIMIT:
 	  return sc_flt(c[o], 0, 0, "ratelimit");
 	case OPT_NETWORK_TIMEOUT:
@@ -426,10 +427,14 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	  return sc_int(c[o], 0, 0, "client_is_windows");
 	case OPT_PEER_VERSION:
 	  return sc_str(c[o], 0, 0, "peer_version");
-	case OPT_ADDRESS:
-	  return sc_str(c[o], 0, 0, "address");
 	case OPT_PORT:
 	  return sc_lst(c[o], 0, 0, "port");
+	case OPT_STATUS_PORT:
+	  return sc_lst(c[o], 0, 0, "status_port");
+	case OPT_LISTEN:
+	  return sc_lst(c[o], 0, 0, "listen");
+	case OPT_LISTEN_STATUS:
+	  return sc_lst(c[o], 0, 0, "listen_status");
 	case OPT_PORT_BACKUP:
 	  return sc_int(c[o], 0, 0, "port_backup");
 	case OPT_PORT_RESTORE:
@@ -440,10 +445,6 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	  return sc_int(c[o], 0, 0, "port_list");
 	case OPT_PORT_DELETE:
 	  return sc_int(c[o], 0, 0, "port_delete");
-	case OPT_STATUS_ADDRESS:
-	  return sc_str(c[o], 0, 0, "status_address");
-	case OPT_STATUS_PORT:
-	  return sc_lst(c[o], 0, 0, "status_port");
 	case OPT_SSL_DHFILE:
 	  return sc_str(c[o], 0, 0, "ssl_dhfile");
 	case OPT_MAX_CHILDREN:
@@ -722,6 +723,12 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	case OPT_WORKING_DIR_RECOVERY_METHOD:
 	  return sc_rec(c[o], RECOVERY_METHOD_DELETE,
 		CONF_FLAG_CC_OVERRIDE, "working_dir_recovery_method");
+	case OPT_MAX_RESUME_ATTEMPTS:
+	  return sc_int(c[o], 0,
+		CONF_FLAG_CC_OVERRIDE, "max_resume_attempts");
+	case OPT_FAIL_ON_WARNING:
+	  return sc_int(c[o], 0,
+		CONF_FLAG_CC_OVERRIDE, "fail_on_warning");
 	case OPT_RSHASH:
 	  return sc_rsh(c[o], RSHASH_UNSET,
 		CONF_FLAG_CC_OVERRIDE, "");
@@ -764,6 +771,12 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	case OPT_EXCREG:
 	  return sc_lst(c[o], 0,
 		CONF_FLAG_INCEXC|CONF_FLAG_STRLIST_SORTED, "exclude_regex");
+	case OPT_INCLOGIC:
+	  return sc_lst(c[o], 0,
+		CONF_FLAG_INCEXC|CONF_FLAG_STRLIST_SORTED, "include_logic");
+	 case OPT_EXCLOGIC:
+	  return sc_lst(c[o], 0,
+		CONF_FLAG_INCEXC|CONF_FLAG_STRLIST_SORTED, "exclude_logic");
 	case OPT_EXCFS:
 	  return sc_lst(c[o], 0,
 		CONF_FLAG_INCEXC|CONF_FLAG_STRLIST_SORTED, "exclude_fs");
@@ -877,6 +890,8 @@ static char *conf_data_to_str(struct conf *conf)
 {
 	size_t l=256;
 	char *ret=NULL;
+	if(!conf->field || !*conf->field)
+		return NULL;
 	if(!(ret=(char *)calloc_w(1, l, __func__))) return ret;
 	*ret='\0';
 	switch(conf->conf_type)
@@ -932,7 +947,8 @@ static char *conf_data_to_str(struct conf *conf)
 				get_mode_t(conf));
 			break;
 		case CT_SSIZE_T:
-			// FIX THIS
+			snprintf(ret, l, "%32s: %" PRIu64 "\n", conf->field,
+				get_uint64_t(conf));
 			break;
 		case CT_CNTR:
 			break;
