@@ -13,6 +13,13 @@
 #include "client/glob_windows.h"
 #include "conffile.h"
 
+static struct strlist *cli_overrides=NULL;
+
+void conf_set_cli_overrides(struct strlist *overrides)
+{
+	cli_overrides=overrides;
+}
+
 // This will strip off everything after the last quote. So, configs like this
 // should work:
 // exclude_regex = "[A-Z]:/pagefile.sys" # swap file (Windows XP, 7, 8)
@@ -22,12 +29,16 @@ static int remove_quotes(const char *f, char **v, char quote)
 	char *dp=NULL;
 	char *sp=NULL;
 	char *copy=NULL;
+	int ret=1;
 
 	// If it does not start with a quote, leave it alone.
 	if(**v!=quote) return 0;
 
 	if(!(copy=strdup_w(*v, __func__)))
-		return -1;
+	{
+		ret=-1;
+		goto end;
+	}
 
 	for(dp=*v, sp=copy+1; *sp; sp++)
 	{
@@ -39,7 +50,7 @@ static int remove_quotes(const char *f, char **v, char quote)
 			// Do not complain about trailing comments.
 			if(*sp && *sp!='#')
 				logp("ignoring trailing characters after quote in config '%s = %s'\n", f, copy);
-			return 1;
+			goto end;
 		}
 		else if(*sp=='\\')
 		{
@@ -58,7 +69,9 @@ static int remove_quotes(const char *f, char **v, char quote)
 	}
 	logp("Did not find closing quote in config '%s = %s'\n", f, copy);
 	*dp='\0';
-	return 1;
+end:
+	free_w(&copy);
+	return ret;
 }
 
 // Get field and value pair.
@@ -116,7 +129,7 @@ static int conf_error(const char *conf_path, int line)
 	return -1;
 }
 
-static int get_file_size(const char *v, uint64_t *dest, const char *conf_path, int line)
+int get_file_size(const char *v, uint64_t *dest, const char *conf_path, int line)
 {
 	// Store in bytes, allow k/m/g.
 	const char *cp=NULL;
@@ -424,25 +437,9 @@ static void burp_ca_conf_problem(const char *conf_path,
 	conf_problem(conf_path, msg, r);
 }
 
-#ifdef HAVE_IPV6
-// These should work for IPv4 connections too.
-#define DEFAULT_ADDRESS_MAIN	"::"
-#else
-// Fall back to IPv4 address if IPv6 is not compiled in.
-#define DEFAULT_ADDRESS_MAIN	"0.0.0.0"
-#endif
-
-#define DEFAULT_ADDRESS_STATUS	"localhost"
-
 static int server_conf_checks(struct conf **c, const char *path, int *r)
 {
-	if(!get_strlist(c[OPT_PORT]))
-		conf_problem(path, "port unset", r);
-
 	// FIX THIS: Most of this could be done by flags.
-	if(!get_string(c[OPT_ADDRESS])
-	  && set_string(c[OPT_ADDRESS], DEFAULT_ADDRESS_MAIN))
-			return -1;
 	if(!get_string(c[OPT_DIRECTORY]))
 		conf_problem(path, "directory unset", r);
 	if(!get_string(c[OPT_DEDUP_GROUP]))
@@ -456,11 +453,6 @@ static int server_conf_checks(struct conf **c, const char *path, int *r)
 	if(get_string(c[OPT_ENCRYPTION_PASSWORD]))
 		conf_problem(path,
 		  "encryption_password should not be set on the server!", r);
-	if(!get_string(c[OPT_STATUS_ADDRESS])
-	  && set_string(c[OPT_STATUS_ADDRESS], DEFAULT_ADDRESS_STATUS))
-			return -1;
-	if(!get_strlist(c[OPT_STATUS_PORT])) // carry on if not set.
-		logp("%s: status_port unset", path);
 	if(!get_strlist(c[OPT_KEEP]))
 		conf_problem(path, "keep unset", r);
 	if(get_int(c[OPT_MAX_HARDLINKS])<2)
@@ -713,13 +705,11 @@ static int client_conf_checks(struct conf **c, const char *path, int *r)
 	}
 	if(!get_string(c[OPT_SERVER]))
 		conf_problem(path, "server unset", r);
-	if(!get_strlist(c[OPT_STATUS_PORT])) // carry on if not set.
-		logp("%s: status_port unset\n", path);
 	if(!get_string(c[OPT_SSL_PEER_CN]))
 	{
 		const char *server=get_string(c[OPT_SERVER]);
 		logp("ssl_peer_cn unset\n");
-		if(!server)
+		if(server)
 		{
 			logp("falling back to '%s'\n", server);
 			if(set_string(c[OPT_SSL_PEER_CN], server))
@@ -810,6 +800,12 @@ static int finalise_incexc_dirs(struct conf **c)
 		if(incexc_munge(c, s)) return -1;
 	for(s=get_strlist(c[OPT_EXCLUDE]); s; s=s->next)
 		if(incexc_munge(c, s)) return -1;
+	if(get_strlist(c[OPT_INCREG]) &&
+	   !(get_strlist(c[OPT_INCLUDE]) || get_strlist(c[OPT_INCGLOB])))
+	{
+		logp("Need at least one 'include' or 'include_glob' for the 'include_regex' to work.\n");
+		return -1;
+	}
 	return 0;
 }
 
@@ -872,6 +868,14 @@ static int finalise_start_dirs(struct conf **c)
 	return 0;
 }
 
+static int finalise_fschg_dirs(struct conf **c)
+{
+	struct strlist *s;
+	for(s=get_strlist(c[OPT_FSCHGDIR]); s; s=s->next)
+		strip_trailing_slashes(&s->path);
+	return 0;
+}
+
 // The glob stuff should only run on the client side.
 static int finalise_glob(struct conf **c)
 {
@@ -908,7 +912,9 @@ int reeval_glob(struct conf **c)
 		return -1;
 
 	if(finalise_incexc_dirs(c)
-	  || finalise_start_dirs(c)) return -1;
+	  || finalise_start_dirs(c)
+	  || finalise_fschg_dirs(c))
+		return -1;
 
 	return 0;
 }
@@ -970,38 +976,68 @@ static int setup_script_arg_overrides(struct conf *c,
 	  || setup_script_arg_override(c, post_args);
 }
 
-static int finalise_server_ports(struct conf **c,
-	enum conf_opt port_opt, enum conf_opt max_children_opt)
+static int listen_config_ok(const char *l)
 {
-	struct strlist *p;
+	int port;
+	const char *c=NULL;
+	const char *cp=NULL;
+
+	for(c=l; *c; c++)
+		if(!isalnum(*c) && *c!=':' && *c!='.')
+			return 0;
+
+	if(!(cp=strrchr(l, ':')))
+		return 0;
+	if(l==cp)
+		return 0;
+	cp++;
+	if(!strlen(cp) || strlen(cp)>5)
+		return 0;
+	port=atoi(cp);
+	if(port<=0 || port>65535)
+		return 0;
+
+	return 1;
+}
+
+static int finalise_server_max_children(struct conf **c,
+	enum conf_opt listen_opt, enum conf_opt max_children_opt)
+{
+	struct strlist *l;
 	struct strlist *mc;
 	long max_children=5;
 
-	for(p=get_strlist(c[port_opt]),
-	   mc=get_strlist(c[max_children_opt]); p; p=p->next)
+	for(l=get_strlist(c[listen_opt]),
+	   mc=get_strlist(c[max_children_opt]); l; l=l->next)
 	{
+		if(!listen_config_ok(l->path))
+		{
+			logp("Could not parse %s config '%s'\n",
+				c[listen_opt]->field, l->path);
+			return -1;
+		}
 		if(mc)
 		{
 			if((max_children=atol(mc->path))<=0)
 			{
 				logp("%s too low for %s %s\n",
 					c[max_children_opt]->field,
-					c[port_opt]->field,
-					p->path);
+					c[listen_opt]->field,
+					l->path);
 				return -1;
 			}
-			p->flag=max_children;
-	
+			l->flag=max_children;
+
 			mc=mc->next;
 		}
 		else
 		{
 			logp("%s %s defaulting to %s %lu\n",
-				c[port_opt]->field,
-				p->path,
+				c[listen_opt]->field,
+				l->path,
 				c[max_children_opt]->field,
 				max_children);
-			p->flag=max_children;
+			l->flag=max_children;
 		}
 	}
 
@@ -1038,12 +1074,45 @@ static int finalise_client_ports(struct conf **c)
 	return 0;
 }
 
+static int apply_cli_overrides(struct conf **confs)
+{
+	int ret=-1;
+	int line=0;
+	char *opt=NULL;
+	struct strlist *oo=NULL;
+
+	for(oo=cli_overrides; oo; oo=oo->next)
+	{
+		line++;
+		free_w(&opt);
+		if(!(opt=strdup_w(oo->path, __func__)))
+			goto end;
+		if((ret=conf_parse_line(confs, "", opt, line)))
+		{
+			logp("Unable to parse cli option %d '%s'\n",
+				line, oo->path);
+			goto end;
+		}
+	}
+	ret=0;
+end:
+	free_w(&opt);
+	return ret;
+}
+
 static int conf_finalise(struct conf **c)
 {
-	enum burp_mode burp_mode=get_e_burp_mode(c[OPT_BURP_MODE]);
+	enum burp_mode burp_mode;
 	int s_script_notify=0;
-	if(finalise_fstypes(c, OPT_EXCFS)) return -1;
-	if(finalise_fstypes(c, OPT_INCFS)) return -1;
+
+	if(apply_cli_overrides(c))
+		return -1;
+
+	burp_mode=get_e_burp_mode(c[OPT_BURP_MODE]);
+
+	if(finalise_fstypes(c, OPT_EXCFS)
+	  || finalise_fstypes(c, OPT_INCFS))
+		return -1;
 
 	strlist_compile_regexes(get_strlist(c[OPT_INCREG]));
 	strlist_compile_regexes(get_strlist(c[OPT_EXCREG]));
@@ -1053,19 +1122,28 @@ static int conf_finalise(struct conf **c)
 	set_max_ext(get_strlist(c[OPT_EXCOM]));
 
 	if(burp_mode==BURP_MODE_CLIENT
-	  && finalise_glob(c)) return -1;
+	  && finalise_glob(c))
+		return -1;
 
 	if(finalise_incexc_dirs(c)
-	  || finalise_start_dirs(c)) return -1;
+	  || finalise_start_dirs(c)
+	  || finalise_fschg_dirs(c))
+		return -1;
 
-	if(finalise_keep_args(c)) return -1;
+	if(finalise_keep_args(c))
+		return -1;
 
 	if(burp_mode==BURP_MODE_SERVER)
 	{
-		if(finalise_server_ports(c,
-			OPT_PORT, OPT_MAX_CHILDREN)
-		  || finalise_server_ports(c,
-			OPT_STATUS_PORT, OPT_MAX_STATUS_CHILDREN))
+		if(!get_strlist(c[OPT_LISTEN]))
+		{
+			logp("Need at least one 'listen' config.\n");
+			return -1;
+		}
+		if(finalise_server_max_children(c,
+			OPT_LISTEN, OPT_MAX_CHILDREN)
+		  || finalise_server_max_children(c,
+			OPT_LISTEN_STATUS, OPT_MAX_STATUS_CHILDREN))
 				return -1;
 	}
 	if(burp_mode==BURP_MODE_CLIENT)
@@ -1213,6 +1291,65 @@ int conf_parse_incexcs_buf(struct conf **c, const char *incexc)
 	return 0;
 }
 
+/* The client runs this when the server overrides the incexcs for restore. */
+int conf_parse_incexcs_srestore(struct conf **c, const char *incexc)
+{
+	int ret=-1;
+	char *rp=NULL;
+	char *oldprefix=NULL;
+	char *srvprefix=NULL;
+	char *newprefix=NULL;
+	const char *rpfield=c[OPT_RESTOREPREFIX]->field;
+
+	if(!(rp=get_string(c[OPT_RESTOREPREFIX])))
+	{
+		logp("The client side must specify a %s!\n", rpfield);
+		goto end;
+	}
+	if(!(oldprefix=strdup_w(rp, __func__)))
+		goto end;
+
+	free_incexcs(c);
+	if(conf_load_lines_from_buf(incexc, c)
+	  || conf_finalise(c))
+		goto end;
+
+	if((srvprefix=get_string(c[OPT_RESTOREPREFIX])))
+	{
+		if(has_dot_component(srvprefix))
+		{
+			logp("The server gave %s '%s', which is not allowed!",
+				rpfield, srvprefix);
+			goto end;
+		}
+		if(!strcmp(oldprefix, "/"))
+		{
+			// Avoid double slash.
+			if(!(newprefix=prepend_s("", srvprefix)))
+				goto end;
+		}
+		else
+		{
+			if(!(newprefix=prepend_s(oldprefix, srvprefix)))
+				goto end;
+		}
+		if(set_string(c[OPT_RESTOREPREFIX], newprefix))
+			goto end;
+		if(build_path_w(newprefix))
+			goto end;
+	}
+	else
+	{
+		if(set_string(c[OPT_RESTOREPREFIX], oldprefix))
+			goto end;
+	}
+	ret=0;
+end:
+	free_w(&oldprefix);
+	free_w(&newprefix);
+	return ret;
+}
+
 static int conf_set_from_global(struct conf **globalc, struct conf **cc)
 {
 	int i=0;
@@ -1356,6 +1493,18 @@ int conf_load_overrides(struct conf **globalcs, struct conf **cconfs,
 	return do_conf_load_overrides(globalcs, cconfs, path, NULL);
 }
 
+int cname_valid(const char *cname)
+{
+	if(!cname) return 0;
+	if(cname[0]=='.'
+	  || strchr(cname, '/') // Avoid path attacks.
+	  || strchr(cname, '\\') // Be cautious of backslashes too.
+	  // I am told that emacs tmp files end with '~'.
+	  || cname[strlen(cname)-1]=='~')
+		return 0;
+	return 1;
+}
+
 int conf_load_clientconfdir(struct conf **globalcs, struct conf **cconfs)
 {
 	int ret=-1;
@@ -1364,9 +1513,9 @@ int conf_load_clientconfdir(struct conf **globalcs, struct conf **cconfs)
 
 	if(conf_init_save_cname_and_version(cconfs)) goto end;
 	cname=get_string(cconfs[OPT_CNAME]);
-	if(looks_like_tmp_or_hidden_file(cname))
+	if(!cname_valid(cname))
 	{
-		logp("client name '%s' is invalid\n", cname);
+		logp("client name '%s' is not valid\n", cname);
 		goto end;
 	}
 
