@@ -180,7 +180,7 @@ static int is_printable(struct iobuf *iobuf)
 {
 	size_t l;
 	for(l=0; l<iobuf->len; l++)
-		if(!isprint(iobuf->buf[l]))
+		if(!isprint(iobuf->buf[l]) && iobuf->buf[l]!='\n')
 			return 0;
 	return 1;
 }
@@ -196,3 +196,12 @@ const char *iobuf_to_printable(struct iobuf *iobuf)
 			"%c:%04X:(binary data)", iobuf->cmd, (int)iobuf->len);
 	return str;
 }
+
+int iobuf_relative_path_attack(struct iobuf *iobuf)
+{
+	if(!has_dot_component(iobuf->buf))
+		return 0;
+	iobuf_log_unexpected(iobuf, __func__);
+	return 1;
+}
+
