@@ -169,7 +169,7 @@ static int send_features(struct asfd *asfd, struct conf **cconfs,
 			goto end;
 	}
 
-#ifndef RS_DEFAULT_STRONG_LEN
+#ifdef HAVE_BLAKE2
 	if(append_to_feat(&feat, "rshash=blake2:"))
 		goto end;
 #endif
@@ -202,6 +202,11 @@ static int do_autoupgrade(struct asfd *asfd, struct vers *vers,
 	ret=0;
 	if(os && *os)
 	{
+		// Sanitise path separators
+		for(char *i=os; *i; ++i)
+			if(*i == '/' || *i == '\\' || *i == ':')
+				*i='-';
+
 		ret=autoupgrade_server(asfd, vers->ser,
 			vers->cli, os, get_cntr(globalcs),
 			autoupgrade_dir);
@@ -280,7 +285,8 @@ static int extra_comms_read(struct async *as,
 			const char *restore_path=get_string(
 				cconfs[OPT_RESTORE_PATH]);
 			// Client will not accept the restore.
-			unlink(restore_path);
+			if (restore_path)
+				unlink(restore_path);
 			if(set_string(cconfs[OPT_RESTORE_PATH], NULL))
 				goto end;
 			logp("Client not accepting server initiated restore.\n");
@@ -393,12 +399,12 @@ static int extra_comms_read(struct async *as,
 		}
 		else if(!strncmp_w(rbuf->buf, "rshash=blake2"))
 		{
-#ifdef RS_DEFAULT_STRONG_LEN
-			logp("Client is trying to use librsync hash blake2, but server does not support it.\n");
-			goto end;
-#else
+#ifdef HAVE_BLAKE2
 			set_e_rshash(cconfs[OPT_RSHASH], RSHASH_BLAKE2);
 			set_e_rshash(globalcs[OPT_RSHASH], RSHASH_BLAKE2);
+#else
+			logp("Client is trying to use librsync hash blake2, but server does not support it.\n");
+			goto end;
 #endif
 		}
 		else if(!strncmp_w(rbuf->buf, "msg"))
