@@ -156,8 +156,7 @@ int cstat_get_client_names(struct cstat **clist, const char *clientconfdir)
 	}
 	for(i=0; i<n; i++)
 	{
-		// looks_like...() also avoids '.' and '..'.
-		if(looks_like_tmp_or_hidden_file(dir[i]->d_name))
+		if(!cname_valid(dir[i]->d_name))
 			continue;
 		for(c=*clist; c; c=c->next)
 		{
@@ -332,7 +331,7 @@ int reload_from_clientdir(struct cstat **clist)
 // FIX THIS: should probably not load everything each time.
 //		if(bu_get_current(sdirs, &c->bu))
 //			goto error;
-		if(bu_get_list_with_working(sdirs, &c->bu, c))
+		if(bu_get_list_with_working(sdirs, &c->bu))
 			goto error;
 		reloaded++;
 	}
@@ -362,7 +361,7 @@ int cstat_set_backup_list(struct cstat *cstat)
 
 	if(!cstat->permitted) return 0;
 
-	if(bu_get_list_with_working((struct sdirs *)cstat->sdirs, &bu, cstat))
+	if(bu_get_list_with_working((struct sdirs *)cstat->sdirs, &bu))
 	{
 		//logp("error when looking up current backups\n");
 		return 0;
