@@ -180,14 +180,11 @@ static enum processed_e process_changed_file(struct asfd *asfd,
 
 	blocklen=get_librsync_block_len(cb->endfile.buf);
 	if(!(p1b->protocol1->sigjob=
-#ifdef RS_DEFAULT_STRONG_LEN
-		rs_sig_begin(blocklen, RS_DEFAULT_STRONG_LEN)
-#else
-		// This is for librsync-1.0.0. RS_DEFAULT_STRONG_LEN was 8 in
-		// librsync-0.9.7.
-		rs_sig_begin(blocklen, 8,
-		  rshash_to_magic_number(get_e_rshash(cconfs[OPT_RSHASH])))
+		rs_sig_begin(blocklen, PROTO1_RS_STRONG_LEN
+#ifndef RS_DEFAULT_STRONG_LEN
+		  , rshash_to_magic_number(get_e_rshash(cconfs[OPT_RSHASH]))
 #endif
+		)
 	))
 	{
 		logp("could not start signature job.\n");
@@ -831,6 +828,13 @@ static enum str_e do_stuff_to_receive(struct asfd *asfd,
 					chmanio, cconfs, last_requested))
 						goto error;
 				return STR_OK;
+			case CMD_INTERRUPT:
+				if(*last_requested
+				  && !strcmp(rbuf->buf, *last_requested))
+					free_w(last_requested);
+				fzp_close(&(rb->protocol1->fzp));
+				sbuf_free_content(rb);
+				return STR_OK;
 			default:
 				iobuf_log_unexpected(rbuf, __func__);
 				goto error;
@@ -841,6 +845,8 @@ static enum str_e do_stuff_to_receive(struct asfd *asfd,
 	switch(rbuf->cmd)
 	{
 		case CMD_DATAPTH:
+			if(iobuf_relative_path_attack(rbuf))
+				goto error;
 			iobuf_move(&rb->protocol1->datapth, rbuf);
 			return STR_OK;
 		case CMD_ATTRIBS:
@@ -1096,6 +1102,8 @@ int backup_phase2_server_protocol1(struct async *as, struct sdirs *sdirs,
 	int breaking=0;
 	int breakcount=0;
 	struct cntr *cntr=NULL;
+	struct cntr_ent *warn_ent=NULL;
+	int fail_on_warning=0;
 
 	if(!as)
 	{
@@ -1119,6 +1127,9 @@ int backup_phase2_server_protocol1(struct async *as, struct sdirs *sdirs,
 		goto error;
 	}
 	cntr=get_cntr(cconfs);
+	fail_on_warning=get_int(cconfs[OPT_FAIL_ON_WARNING]);
+	if(cntr)
+		warn_ent=cntr->ent[CMD_WARNING];
 
 	if(get_int(cconfs[OPT_BREAKPOINT])>=2000
 	  && get_int(cconfs[OPT_BREAKPOINT])<3000)
@@ -1180,6 +1191,9 @@ int backup_phase2_server_protocol1(struct async *as, struct sdirs *sdirs,
 
 	while(1)
 	{
+		if(check_fail_on_warning(fail_on_warning, warn_ent))
+			goto error;
+
 		if(breaking && breakcount--==0)
 			return breakpoint(breaking, __func__);
 
@@ -1204,6 +1218,9 @@ int backup_phase2_server_protocol1(struct async *as, struct sdirs *sdirs,
 				case STR_OK:
 					break;
 				case STR_FINISHED:
+					if(check_fail_on_warning(
+						fail_on_warning, warn_ent))
+							goto error;
 					goto end;
 				case STR_ERROR:
 					goto error;
@@ -1229,6 +1246,9 @@ int backup_phase2_server_protocol1(struct async *as, struct sdirs *sdirs,
 
 error:
 	ret=-1;
+	if(p1b)
+		logp("  last tried file:    %s\n",
+			iobuf_to_printable(&p1b->path));
 end:
 	if(manio_close(&chmanio))
 	{
