@@ -60,11 +60,6 @@ end:
 	return result;
 }
 
-// Help librsync-1.0.0
-#ifndef RS_DEFAULT_STRONG_LEN
-#define RS_DEFAULT_STRONG_LEN	8
-#endif
-
 static int make_rev_sig(const char *dst, const char *sig, const char *endfile,
 	int compression, struct conf **confs)
 {
@@ -82,7 +77,7 @@ static int make_rev_sig(const char *dst, const char *sig, const char *endfile,
 	  || !(sigp=fzp_open(sig, "wb"))
 	  || rs_sig_gzfile(dstfzp, sigp,
 		get_librsync_block_len(endfile),
-		RS_DEFAULT_STRONG_LEN, confs)!=RS_DONE)
+		PROTO1_RS_STRONG_LEN, confs)!=RS_DONE)
 			goto end;
 	ret=0;
 end:
