@@ -17,17 +17,19 @@ struct rblk
 {
 	uint64_t hash_key;
 	struct iobuf readbuf[DATA_FILE_SIG_MAX];
-	uint16_t readbuflen;
+	uint16_t rlen;
+	struct fzp *fzp;
 	UT_hash_handle hh;
 };
 
 static void rblk_free_content(struct rblk *rblk)
 {
-	for(int j=0; j<DATA_FILE_SIG_MAX; j++)
+	for(int j=0; j<rblk->rlen; j++)
 	{
 		rblk_mem-=rblk->readbuf[j].len;
 		iobuf_free_content(&rblk->readbuf[j]);
 	}
+	fzp_close(&rblk->fzp);
 }
 
 static void rblk_free(struct rblk **rblk)
@@ -35,8 +37,8 @@ static void rblk_free(struct rblk **rblk)
 /*
 uint16_t datno;
 char *x;
-x=uint64_to_savepathstr_with_sig_uint((*rblk)->savepath, &datno);
-printf(" free: %s\n", x);
+x=uint64_to_savepathstr_with_sig_uint((*rblk)->hash_key, &datno);
+logp("close: %s\n", x);
 */
 	rblk_free_content(*rblk);
 	rblk_mem-=sizeof(struct rblk);
@@ -84,7 +86,7 @@ void rblks_free(void)
 	rblk_hash=NULL;
 }
 
-static int rblks_free_one(void)
+static int rblks_free_one_except(struct rblk *keep)
 {
 	uint64_t before=rblk_mem;
 	struct rblk *tmp;
@@ -92,6 +94,8 @@ static int rblks_free_one(void)
 
 	HASH_ITER(hh, rblk_hash, rblk, tmp)
 	{
+		if(rblk==keep)
+			continue;
 		HASH_DEL(rblk_hash, rblk);
 		rblk_free(&rblk);
 		break;
@@ -101,45 +105,6 @@ static int rblks_free_one(void)
 	return -1;
 }
 
-static int rblk_load_from_disk(struct rblk *rblk, const char *fulldatpath)
-{
-	int r;
-	int ret=-1;
-	int done=0;
-	struct fzp *fzp=NULL;
-	struct iobuf rbuf;
-
-	iobuf_init(&rbuf);
-
-	if(!(fzp=fzp_open(fulldatpath, "rb")))
-		goto end;
-	for(r=0; r<DATA_FILE_SIG_MAX; r++)
-	{
-		switch(iobuf_fill_from_fzp_data(&rbuf, fzp))
-		{
-			case 0: if(rbuf.cmd!=CMD_DATA)
-				{
-					logp("unknown cmd in %s: %c\n",
-						__func__, rbuf.cmd);
-					goto end;
-				}
-				iobuf_free_content(&rblk->readbuf[r]);
-				iobuf_move(&rblk->readbuf[r], &rbuf);
-				rblk_mem+=rblk->readbuf[r].len;
-				continue;
-			case 1: done++;
-				break;
-			default: goto end;
-		}
-		if(done) break;
-	}
-	rblk->readbuflen=r;
-	ret=0;
-end:
-	fzp_close(&fzp);
-	return ret;
-}
-
 static int rblk_init(struct rblk *rblk, struct blk *blk,
 	uint64_t hash_key, const char *datpath, const char *savepathstr)
 {
@@ -149,8 +114,8 @@ static int rblk_init(struct rblk *rblk, struct blk *blk,
 	rblk->hash_key=hash_key;
 	if(!(fulldatpath=prepend_s(datpath, savepathstr)))
 		goto end;
-	logp("load: %s\n", savepathstr);
-	if(rblk_load_from_disk(rblk, fulldatpath))
+	logp("open: %s\n", savepathstr);
+	if(!(rblk->fzp=fzp_open(fulldatpath, "rb")))
 		goto end;
 	ret=0;
 end:
@@ -158,6 +123,45 @@ end:
 	return ret;
 }
 
+static int rblk_load_more_chunks(struct rblk *rblk, uint16_t datno_target)
+{
+	int ret=-1;
+	int done=0;
+	struct iobuf rbuf;
+
+	iobuf_init(&rbuf);
+
+	for(
+		;
+		rblk->rlen<DATA_FILE_SIG_MAX && rblk->rlen<=datno_target;
+		rblk->rlen++
+	) {
+		switch(iobuf_fill_from_fzp_data(&rbuf, rblk->fzp))
+		{
+			case 0:
+				if(rbuf.cmd!=CMD_DATA)
+				{
+					logp("unknown cmd in %s: %c\n",
+						__func__, rbuf.cmd);
+					goto end;
+				}
+				iobuf_move(&rblk->readbuf[rblk->rlen], &rbuf);
+				rblk_mem+=rblk->readbuf[rblk->rlen].len;
+				continue;
+			case 1:
+				done++;
+				break;
+			default:
+				goto end;
+		}
+		if(done)
+			break;
+	}
+	ret=0;
+end:
+	return ret;
+}
+
 int rblk_retrieve_data(struct asfd *asfd, struct cntr *cntr,
 	struct blk *blk, const char *datpath)
 {
@@ -177,23 +181,30 @@ int rblk_retrieve_data(struct asfd *asfd, struct cntr *cntr,
 			rblk_free(&rblk);
 			return -1;
 		}
-		while(rblk_mem>rblk_mem_max)
-		{
-			if(rblks_free_one())
-			{
-				logw(asfd, cntr,
-					"rblk_memory_max is too low!\n");
-				break;
-			}
-		}
 		rblk_hash_add(rblk);
 	}
 
-// printf("lookup: %s (%u)\n", savepathstr, datno);
-	if(datno>rblk->readbuflen)
+	if(datno>=rblk->rlen)
 	{
-		logp("dat index %d is greater than readbuflen: %d\n",
-			datno, rblk->readbuflen);
+		// Need to load more from this data file.
+		if(rblk_load_more_chunks(rblk, datno))
+			return -1;
+	}
+
+	while(rblk_mem>rblk_mem_max)
+	{
+		if(rblks_free_one_except(rblk))
+		{
+			logw(asfd, cntr, "rblk_memory_max is too low!\n");
+			break;
+		}
+	}
+
+// printf("lookup: %s (%u)\n", savepathstr, datno);
+	if(datno>=rblk->rlen)
+	{
+		logp("datno %d is greater than rlen: %d\n",
+			datno, rblk->rlen);
 		return -1;
 	}
 	blk->data=rblk->readbuf[datno].buf;
