@@ -211,6 +211,7 @@ static void set_mtimes(const char *cnames[], int diff)
 START_TEST(test_cstat_reload_from_client_confs)
 {
 	struct cstat *clist=NULL;
+	struct conf **monitor_cconfs;
 	struct conf **globalcs;
 	struct conf **cconfs;
 	const char *cnames1[] = {"cli1", NULL};
@@ -218,71 +219,75 @@ START_TEST(test_cstat_reload_from_client_confs)
 	const char *cnames13[] = {"cli1", "cli3", NULL};
 
 	clean();
+	fail_unless((monitor_cconfs=confs_alloc())!=NULL);
 	fail_unless((globalcs=confs_alloc())!=NULL);
 	fail_unless((cconfs=confs_alloc())!=NULL);
+	fail_unless(!confs_init(monitor_cconfs));
 	fail_unless(!confs_init(globalcs));
 	build_file(GLOBAL_CONF, MIN_SERVER_CONF);
+	fail_unless(!conf_load_global_only(GLOBAL_CONF, monitor_cconfs));
 	fail_unless(!conf_load_global_only(GLOBAL_CONF, globalcs));
 	build_clientconfdir_files(cnames123, NULL);
 
 	fail_unless(!cstat_get_client_names(&clist, CLIENTCONFDIR));
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==3);
 	assert_cstat_list(clist, cnames123);
 
 	// Going again should result in 0 updates.
 	fail_unless(!cstat_get_client_names(&clist, CLIENTCONFDIR));
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==0);
 	assert_cstat_list(clist, cnames123);
 
 	// Touching all clientconfdir files should result in 3 updates.
 	set_mtimes(cnames123, -100);
 	fail_unless(!cstat_get_client_names(&clist, CLIENTCONFDIR));
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==3);
 	assert_cstat_list(clist, cnames123);
 
 	// Touching one clientconfdir file should result in 1 update.
 	set_mtimes(cnames1, -200);
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==1);
 	assert_cstat_list(clist, cnames123);
 
 	// Deleting one clientconfdir file should result in 0 updates.
 	delete_clientconfdir_file("cli2");
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==0);
 	assert_cstat_list(clist, cnames13);
 
 	// A clientconfdir file with junk in it should get permission denied
 	// and remain in the list.
 	build_file(get_clientconfdir_path("cli1"), "klasdjldkjf");
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==0);
 	assert_cstat_list(clist, cnames13);
 	// Should not reload it next time round.
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==0);
 	assert_cstat_list(clist, cnames13);
 	// Fix the file, and we should reload it.
 	build_file(get_clientconfdir_path("cli1"), NULL);
 	set_mtimes(cnames1, -300);
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==1);
 	assert_cstat_list(clist, cnames13);
 
 	// Delete everything.
 	delete_clientconfdir_file("cli1");
 	delete_clientconfdir_file("cli3");
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==0);
 	assert_cstat_list(clist, NULL);
 
 	unlink(GLOBAL_CONF);
-	fail_unless(cstat_reload_from_client_confs(&clist,
+	fail_unless(cstat_reload_from_client_confs(&clist, monitor_cconfs,
 		globalcs, cconfs)==-1);
 
+	confs_free(&monitor_cconfs);
 	confs_free(&globalcs);
 	confs_free(&cconfs);
 	cstat_list_free_sdirs(clist);
@@ -292,15 +297,20 @@ START_TEST(test_cstat_reload_from_client_confs)
 }
 END_TEST
 
+static void setup_globalcs(struct conf ***globalcs)
+{
+	fail_unless((*globalcs=confs_alloc())!=NULL);
+	fail_unless(!confs_init(*globalcs));
+	build_file(GLOBAL_CONF, MIN_SERVER_CONF);
+	fail_unless(!conf_load_global_only(GLOBAL_CONF, *globalcs));
+}
+
 static struct cstat *test_cstat_remove_setup(struct conf ***globalcs,
 	const char *cnames[])
 {
 	struct cstat *clist=NULL;
 	clean();
-	fail_unless((*globalcs=confs_alloc())!=NULL);
-	fail_unless(!confs_init(*globalcs));
-	build_file(GLOBAL_CONF, MIN_SERVER_CONF);
-	fail_unless(!conf_load_global_only(GLOBAL_CONF, *globalcs));
+	setup_globalcs(globalcs);
 	build_clientconfdir_files(cnames, NULL);
 	fail_unless(!cstat_get_client_names(&clist, CLIENTCONFDIR));
 	assert_cstat_list(clist, cnames);
@@ -423,7 +433,7 @@ static void test_cstat_set_run_status_not_permitted(enum protocol protocol)
 {
 	struct cstat *cstat;
 	cstat=set_run_status_setup(protocol, 0 /*not permitted*/);
-	cstat_set_run_status(cstat);
+	cstat_set_run_status(cstat, RUN_STATUS_RUNNING);
 	fail_unless(cstat->run_status==RUN_STATUS_UNSET);
 	tear_down(&cstat);
 }
@@ -432,76 +442,17 @@ static void test_cstat_set_run_status_idle(enum protocol protocol)
 {
 	struct cstat *cstat;
 	cstat=set_run_status_setup(protocol, 1 /*permitted*/);
-	cstat_set_run_status(cstat);
+	cstat_set_run_status(cstat, RUN_STATUS_IDLE);
 	fail_unless(cstat->run_status==RUN_STATUS_IDLE);
 	tear_down(&cstat);
 }
 
-static struct sd sd1w[] = {
-	{ "0000001 1970-01-01 00:00:00", 1, 1, BU_WORKING }
-};
-
-static void test_cstat_set_run_status_client_crashed(enum protocol protocol)
-{
-	struct cstat *cstat;
-	cstat=set_run_status_setup(protocol, 1 /*permitted*/);
-	fail_unless(recursive_delete(BASE)==0);
-	build_storage_dirs((struct sdirs *)cstat->sdirs,
-		sd1w, ARR_LEN(sd1w));
-	cstat_set_run_status(cstat);
-	fail_unless(cstat->run_status==RUN_STATUS_CLIENT_CRASHED);
-	tear_down(&cstat);
-}
-
-static void test_cstat_set_run_status_server_crashed(enum protocol protocol)
-{
-	struct cstat *cstat;
-	struct sdirs *sdirs;
-	cstat=set_run_status_setup(protocol, 1 /*permitted*/);
-	fail_unless(recursive_delete(BASE)==0);
-	build_storage_dirs((struct sdirs *)cstat->sdirs,
-		sd1w, ARR_LEN(sd1w));
-	sdirs=(struct sdirs *)cstat->sdirs;
-	build_file(sdirs->lock->path, NULL);
-	cstat_set_run_status(cstat);
-	fail_unless(cstat->run_status==RUN_STATUS_SERVER_CRASHED);
-	tear_down(&cstat);
-}
-
 static void test_cstat_set_run_status_running(enum protocol protocol)
 {
-	int waitstat;
 	struct cstat *cstat;
-	struct sdirs *sdirs;
-	struct lock *lock;
 	cstat=set_run_status_setup(protocol, 1 /*permitted*/);
-	clean();
-	build_storage_dirs((struct sdirs *)cstat->sdirs,
-		sd1w, ARR_LEN(sd1w));
-	sdirs=(struct sdirs *)cstat->sdirs;
-	lock=sdirs->lock;
-
-        switch(fork())
-	{
-		case -1: fail_unless(0==1);
-			 break;
-		case 0: // Child.
-		{
-			lock_get_quick(lock);
-			sleep(2);
-			lock_release(lock);
-			exit(0);
-		}
-		default: break;
-	}
-        // Parent.
-
-	sleep(1);
-	lock_get(lock);
-	fail_unless(lock->status==GET_LOCK_NOT_GOT);
-	cstat_set_run_status(cstat);
+	cstat_set_run_status(cstat, RUN_STATUS_RUNNING);
 	fail_unless(cstat->run_status==RUN_STATUS_RUNNING);
-	wait(&waitstat);
 	tear_down(&cstat);
 }
 
@@ -511,10 +462,6 @@ START_TEST(test_cstat_set_run_status)
 	test_cstat_set_run_status_not_permitted(PROTO_2);
 	test_cstat_set_run_status_idle(PROTO_1);
 	test_cstat_set_run_status_idle(PROTO_2);
-	test_cstat_set_run_status_client_crashed(PROTO_1);
-	test_cstat_set_run_status_client_crashed(PROTO_2);
-	test_cstat_set_run_status_server_crashed(PROTO_1);
-	test_cstat_set_run_status_server_crashed(PROTO_2);
 	test_cstat_set_run_status_running(PROTO_1);
 	test_cstat_set_run_status_running(PROTO_2);
 }
@@ -533,8 +480,6 @@ static void do_test_cstat_reload_from_client_confs(enum protocol protocol)
 
 	// First time, nothing is reloaded.
 	fail_unless(reload_from_clientdir(&clist)==0);
-	fail_unless(c1->run_status==RUN_STATUS_IDLE);
-	fail_unless(c2->run_status==RUN_STATUS_UNSET);
 	fail_unless(c1->bu==NULL);
 	fail_unless(c2->bu==NULL);
 
@@ -542,18 +487,12 @@ static void do_test_cstat_reload_from_client_confs(enum protocol protocol)
 	build_storage_dirs((struct sdirs *)c1->sdirs,
 		sd123, ARR_LEN(sd123));
 	fail_unless(reload_from_clientdir(&clist)==1);
-	fail_unless(c1->run_status==RUN_STATUS_IDLE);
-	fail_unless(c2->run_status==RUN_STATUS_UNSET);
 	fail_unless(c1->bu!=NULL);
 	fail_unless(c2->bu==NULL);
 
 	// Go again, nothing should be reloaded.
 	fail_unless(reload_from_clientdir(&clist)==0);
 
-	// Get a lock.
-	lock_get_quick(((struct sdirs *)c1->sdirs)->lock);
-	fail_unless(reload_from_clientdir(&clist)==1);
-
 	sdirs_free((struct sdirs **)&c1->sdirs);
 	sdirs_free((struct sdirs **)&c2->sdirs);
 	cstat_list_free(&clist);
@@ -569,61 +508,61 @@ START_TEST(test_cstat_reload_from_clientdir)
 END_TEST
 
 static void check_restore_clients(struct cstat *cstat,
-	struct conf **parentconf, const char *restore_clients, int permitted)
+	struct conf **monitor_cconfs, const char *restore_clients, int permitted)
 {
 	struct conf **cconfs=NULL;
 	fail_unless((cconfs=confs_alloc())!=NULL);
 	fail_unless(!confs_init(cconfs));
 	fail_unless(!set_string(cconfs[OPT_CNAME], "cli1"));
 	build_clientconfdir_file("cli1", restore_clients);
-	fail_unless(!conf_load_clientconfdir(parentconf, cconfs));
-	fail_unless(!set_string(parentconf[OPT_CNAME], "cli2"));
-	fail_unless(cstat_permitted(cstat, parentconf, cconfs)==permitted);
+	fail_unless(!conf_load_clientconfdir(monitor_cconfs, cconfs));
+	fail_unless(!set_string(monitor_cconfs[OPT_CNAME], "cli2"));
+	fail_unless(cstat_permitted(cstat, monitor_cconfs, cconfs)==permitted);
 	confs_free(&cconfs);
 }
 
 START_TEST(test_cstat_permitted)
 {
 	struct cstat *cstat=NULL;
-	struct conf **parentconf=NULL;
+	struct conf **monitor_cconfs=NULL;
 
 	clean();
 	fail_unless((cstat=cstat_alloc())!=NULL);
 	fail_unless(!cstat_init(cstat, "cli1", CLIENTCONFDIR));
-	fail_unless((parentconf=confs_alloc())!=NULL);
-	fail_unless(!confs_init(parentconf));
+	fail_unless((monitor_cconfs=confs_alloc())!=NULL);
+	fail_unless(!confs_init(monitor_cconfs));
 	build_file(GLOBAL_CONF, MIN_SERVER_CONF);
-	fail_unless(!conf_load_global_only(GLOBAL_CONF, parentconf));
+	fail_unless(!conf_load_global_only(GLOBAL_CONF, monitor_cconfs));
 
 	// Clients can look at themselves.
 	// In this case, cli1 is 'us'.
-	fail_unless(!set_string(parentconf[OPT_CNAME], "cli1"));
-	fail_unless(cstat_permitted(cstat, parentconf, NULL)==1);
+	fail_unless(!set_string(monitor_cconfs[OPT_CNAME], "cli1"));
+	fail_unless(cstat_permitted(cstat, monitor_cconfs, NULL)==1);
 
 	// Clients using the restore_client cannot see anything but the client
 	// they are pretending to be.
 	// In this case, cli2 is 'us'.
-	fail_unless(!set_string(parentconf[OPT_CNAME], "cli2"));
-	fail_unless(!set_string(parentconf[OPT_RESTORE_CLIENT], "is_set"));
-	fail_unless(cstat_permitted(cstat, parentconf, NULL)==0);
-	fail_unless(!set_string(parentconf[OPT_RESTORE_CLIENT], NULL));
+	fail_unless(!set_string(monitor_cconfs[OPT_CNAME], "cli2"));
+	fail_unless(!set_string(monitor_cconfs[OPT_RESTORE_CLIENT], "is_set"));
+	fail_unless(cstat_permitted(cstat, monitor_cconfs, NULL)==0);
+	fail_unless(!set_string(monitor_cconfs[OPT_RESTORE_CLIENT], NULL));
 
 	// Clients can see another client if we are listed in its
 	// restore_clients list.
 	// In this case, cli2 is 'us' and we are trying to look at cli1.
-	check_restore_clients(cstat, parentconf,
+	check_restore_clients(cstat, monitor_cconfs,
 		"restore_client = cli3\n"
 		"restore_client = cli2\n"
 		"restore_client = cli4\n",
 		1 /* permitted */);
 
 	// If we are not in its list, we cannot see it.
-	check_restore_clients(cstat, parentconf,
+	check_restore_clients(cstat, monitor_cconfs,
 		"restore_client = cli3\n"
 		"restore_client = cli4\n",
 		0 /* not permitted */);
 
-	confs_free(&parentconf);
+	confs_free(&monitor_cconfs);
 	cstat_free(&cstat);
 	clean();
 	alloc_check();
@@ -633,12 +572,15 @@ END_TEST
 START_TEST(test_cstat_load_data_from_disk)
 {
 	struct cstat *clist=NULL;
+	struct conf **monitor_cconfs;
 	struct conf **globalcs;
 	struct conf **cconfs;
+	setup_globalcs(&monitor_cconfs);
 	clist=test_cstat_remove_setup(&globalcs, cnames1234);
 	fail_unless((cconfs=confs_alloc())!=NULL);
-	cstat_load_data_from_disk(&clist, globalcs, cconfs);
+	cstat_load_data_from_disk(&clist, monitor_cconfs, globalcs, cconfs);
 	confs_free(&cconfs);
+	confs_free(&monitor_cconfs);
 	test_cstat_remove_teardown(&globalcs, &clist);
 }
 END_TEST
