@@ -14,8 +14,6 @@ static void check_default(struct conf **c, enum conf_opt o)
 			break;
 		case OPT_LOCKFILE:
 		case OPT_PIDFILE:
-		case OPT_ADDRESS:
-		case OPT_STATUS_ADDRESS:
 		case OPT_SSL_CERT_CA:
 		case OPT_SSL_CERT:
 		case OPT_SSL_KEY:
@@ -109,6 +107,9 @@ static void check_default(struct conf **c, enum conf_opt o)
 		case OPT_PORT_VERIFY:
 		case OPT_PORT_LIST:
 		case OPT_PORT_DELETE:
+		case OPT_MAX_RESUME_ATTEMPTS:
+		case OPT_FAIL_ON_WARNING:
+		case OPT_SSL_VERIFY_PEER_EARLY:
 			fail_unless(get_int(c[o])==0);
 			break;
 		case OPT_DAEMON:
@@ -178,6 +179,8 @@ static void check_default(struct conf **c, enum conf_opt o)
 		case OPT_EXCEXT:
 		case OPT_INCREG:
 		case OPT_EXCREG:
+		case OPT_INCLOGIC:
+		case OPT_EXCLOGIC:
 		case OPT_EXCFS:
 		case OPT_INCFS:
 		case OPT_EXCOM:
@@ -187,6 +190,8 @@ static void check_default(struct conf **c, enum conf_opt o)
 		case OPT_LABEL:
 		case OPT_PORT:
 		case OPT_STATUS_PORT:
+		case OPT_LISTEN:
+		case OPT_LISTEN_STATUS:
 		case OPT_MAX_CHILDREN:
 		case OPT_MAX_STATUS_CHILDREN:
 			fail_unless(get_strlist(c[o])==NULL);
