@@ -98,6 +98,17 @@ static void assert_strlist(struct strlist **s, const char *path, int flag)
 	*s=(*s)->next;
 }
 
+static void check_listen(struct conf **confs,
+	const char *listen, int max_children,
+	const char *listen_status, int max_status_children)
+{
+	struct strlist *s;
+	s=get_strlist(confs[OPT_LISTEN]);
+	assert_strlist(&s, listen, max_children);
+	s=get_strlist(confs[OPT_LISTEN_STATUS]);
+	assert_strlist(&s, listen_status, max_status_children);
+}
+
 static void check_ports(struct conf **confs,
 	const char *port, int max_children,
 	const char *status_port, int max_status_children)
@@ -259,6 +270,7 @@ START_TEST(test_client_includes_excludes)
 		"include=/a\n"
 		"include=/a/b/c/d\n"
 		"cross_filesystem=/mnt/x\n"
+		"cross_filesystem=/mnt/y/\n"
 	;
 	struct strlist *s;
 	struct conf **confs=NULL;
@@ -294,6 +306,7 @@ START_TEST(test_client_includes_excludes)
 	assert_strlist(&s, "/a/b/c", 0);
 	assert_strlist(&s, "/a/b/c/d", 0);
 	assert_strlist(&s, "/mnt/x", 0);
+	assert_strlist(&s, "/mnt/y", 0);
 	assert_strlist(&s, NULL, 0);
 	tear_down(NULL, &confs);
 }
@@ -366,7 +379,7 @@ START_TEST(test_server_conf)
 	build_file(CONFFILE, MIN_SERVER_CONF);
 	fail_unless(!conf_load_global_only(CONFFILE, confs));
 	fail_unless(get_e_burp_mode(confs[OPT_BURP_MODE])==BURP_MODE_SERVER);
-	check_ports(confs, "1234", 5, "12345", 5);
+	check_listen(confs, "0.0.0.0:1234", 5, "0.0.0.0:12345", 5);
 	ck_assert_str_eq(get_string(confs[OPT_LOCKFILE]), "/lockfile/path");
 	ck_assert_str_eq(get_string(confs[OPT_SSL_CERT]), "/ssl/cert/path");
 	ck_assert_str_eq(get_string(confs[OPT_SSL_CERT_CA]), "/cert_ca/path");
@@ -385,13 +398,13 @@ START_TEST(test_server_port_conf_one_max_child)
 {
 	struct conf **confs=NULL;
 	setup(&confs, NULL);
-	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
-		"port=1234\n"
-		"status_port=12345\n"
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_LISTEN
+		"listen=0.0.0.0:1234\n"
+		"listen_status=0.0.0.0:12345\n"
 		"max_children=12\n"
 		"max_status_children=21\n");
 	fail_unless(!conf_load_global_only(CONFFILE, confs));
-	check_ports(confs, "1234", 12, "12345", 21);
+	check_listen(confs, "0.0.0.0:1234", 12, "0.0.0.0:12345", 21);
 	tear_down(NULL, &confs);
 }
 END_TEST
@@ -400,7 +413,7 @@ START_TEST(test_server_port_conf_too_many_max_child)
 {
 	struct conf **confs=NULL;
 	setup(&confs, NULL);
-	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_LISTEN
 		"port=1234\n"
 		"max_children=12\n"
 		"max_children=21\n");
@@ -415,19 +428,19 @@ START_TEST(test_server_port_conf_few_max_child)
 	struct strlist *s;
 	struct conf **confs=NULL;
 	setup(&confs, NULL);
-	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
-		"port=1234\n"
-		"port=2345\n"
-		"port=3456\n"
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_LISTEN
+		"listen=0.0.0.0:1234\n"
+		"listen=1.1.1.1:2345\n"
+		"listen=9.8.7.6:3456\n"
 		"max_children=12\n"
 		"max_children=21\n");
 	fail_unless(!conf_load_global_only(CONFFILE, confs));
 
-	s=get_strlist(confs[OPT_PORT]);
-	assert_strlist(&s, "1234", 12);
-	assert_strlist(&s, "2345", 21);
-	assert_strlist(&s, "3456", 21); // takes the previous as default
-	fail_unless(get_strlist(confs[OPT_STATUS_PORT])==NULL);
+	s=get_strlist(confs[OPT_LISTEN]);
+	assert_strlist(&s, "0.0.0.0:1234", 12);
+	assert_strlist(&s, "1.1.1.1:2345", 21);
+	assert_strlist(&s, "9.8.7.6:3456", 21); // takes the previous as default
+	fail_unless(get_strlist(confs[OPT_LISTEN_STATUS])==NULL);
 	tear_down(NULL, &confs);
 }
 END_TEST
@@ -437,29 +450,29 @@ START_TEST(test_server_port_conf_complex)
 	struct strlist *s;
 	struct conf **confs=NULL;
 	setup(&confs, NULL);
-	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
-		"port=1234\n"
-		"port=2345\n"
-		"port=3456\n"
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_LISTEN
+		"listen=0.0.0.0:1234\n"
+		"listen=1.1.1.1:2345\n"
+		"listen=9.8.7.6:3456\n"
 		"max_children=12\n"
 		"max_children=21\n"
 		"max_children=37\n"
-		"status_port=987\n"
-		"status_port=654\n"
-		"status_port=321\n"
+		"listen_status=::1:987\n"
+		"listen_status=127.0.0.1:654\n"
+		"listen_status=1abc:2323::9999:321\n"
 		"max_status_children=4\n"
 		"max_status_children=5\n"
 		"max_status_children=6\n");
 	fail_unless(!conf_load_global_only(CONFFILE, confs));
 
-	s=get_strlist(confs[OPT_PORT]);
-	assert_strlist(&s, "1234", 12);
-	assert_strlist(&s, "2345", 21);
-	assert_strlist(&s, "3456", 37);
-	s=get_strlist(confs[OPT_STATUS_PORT]);
-	assert_strlist(&s, "987", 4);
-	assert_strlist(&s, "654", 5);
-	assert_strlist(&s, "321", 6);
+	s=get_strlist(confs[OPT_LISTEN]);
+	assert_strlist(&s, "0.0.0.0:1234", 12);
+	assert_strlist(&s, "1.1.1.1:2345", 21);
+	assert_strlist(&s, "9.8.7.6:3456", 37);
+	s=get_strlist(confs[OPT_LISTEN_STATUS]);
+	assert_strlist(&s, "::1:987", 4);
+	assert_strlist(&s, "127.0.0.1:654", 5);
+	assert_strlist(&s, "1abc:2323::9999:321", 6);
 	tear_down(NULL, &confs);
 }
 END_TEST
@@ -716,6 +729,7 @@ START_TEST(test_clientconfdir_conf)
 		"keep=4\n"
 		"keep=7\n"
 		"working_dir_recovery_method=resume\n"
+		"max_resume_attempts=5\n"
 		"librsync=0\n"
 		"librsync_max_size=10Mb\n"
 		"version_warn=0\n"
@@ -753,6 +767,7 @@ START_TEST(test_clientconfdir_conf)
 	assert_include(&s, NULL);
 	fail_unless(get_e_recovery_method(
 	  cconfs[OPT_WORKING_DIR_RECOVERY_METHOD])==RECOVERY_METHOD_RESUME);
+	fail_unless(get_int(cconfs[OPT_MAX_RESUME_ATTEMPTS])==5);
 	fail_unless(get_int(cconfs[OPT_LIBRSYNC])==0);
 	fail_unless(get_uint64_t(cconfs[OPT_LIBRSYNC_MAX_SIZE])==10485760);
 	fail_unless(get_int(cconfs[OPT_VERSION_WARN])==0);
@@ -832,8 +847,7 @@ START_TEST(test_strlist_reset)
 	struct strlist *s;
 	struct conf **confs=NULL;
 	setup(&confs, NULL);
-	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
-		"port=1234\n"
+	build_file(CONFFILE, MIN_SERVER_CONF
 		"timer_arg = /ignored/timer/arg1\n"
 		"timer_arg = /ignored/timer/arg2\n"
 		"timer_arg := /timer/arg3\n"
@@ -915,6 +929,37 @@ START_TEST(test_conf_switch_to_orig_client_ok)
 }
 END_TEST
 
+struct cndata
+{
+	const char *cname;
+	int valid;
+};
+
+static struct cndata c[] = {
+	{ "testclient", 1 },
+	{ "test.client", 1 },
+	{ "testclient..", 1 },
+	{ ".testclient", 0 },
+	{ "testclient~", 0 },
+	{ ".", 0 },
+	{ "..", 0 },
+	{ "/testclient", 0 },
+	{ "test/client", 0 },
+	{ "test\\client", 0 },
+	{ "/..", 0 },
+	{ "../", 0 },
+	{ "../../testclient", 0 },
+	{ "testclient/../blah", 0 },
+};
+
+START_TEST(test_cname_valid)
+{
+	FOREACH(c)
+	{
+		fail_unless(cname_valid(c[i].cname)==c[i].valid);
+	}
+}
+END_TEST
 
 Suite *suite_conffile(void)
 {
@@ -954,6 +999,7 @@ Suite *suite_conffile(void)
 	tcase_add_test(tc_core, test_clientconfdir_server_script);
 	tcase_add_test(tc_core, test_conf_switch_to_orig_client_fail);
 	tcase_add_test(tc_core, test_conf_switch_to_orig_client_ok);
+	tcase_add_test(tc_core, test_cname_valid);
 
 	suite_add_tcase(s, tc_core);
 
